#ifndef __Ik_H
#define __Ik_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: Ik.h,v 1.15 1997/07/12 06:00:23 steve Exp $"
/*
 * $Log: Ik.h,v $
 * Revision 1.15  1997/07/12 06:00:23  steve
 *  Speed up gray display a bit.
 *
 * Revision 1.14  1997/07/09 18:06:45  steve
 *  Better handling of PNG binary.
 *
 * Revision 1.13  1997/07/08 23:17:48  steve
 *  Rework palette handling.
 *
 * Revision 1.12  1997/06/04 08:30:40  steve
 *  Add support for pbm.
 *
 * Revision 1.11  1997/06/01 21:10:45  steve
 *  Start Windows NT support.
 *
 * Revision 1.10  1997/03/17 21:37:06  steve
 *  Put matrix access in seperate function.
 *
 * Revision 1.9  1997/02/08 03:52:49  steve
 *  Get tk out of Ik.h
 *
 * Revision 1.8  1997/02/08 03:23:31  steve
 *  Make bit order explicit
 *
 * Revision 1.7  1997/02/03 08:02:30  steve
 *  Support binary internal image representation, give
 *  the application access to the raw bits if needed,
 *  and allow the X display to share the raw bits if
 *  the encoding matches.
 *
 * Revision 1.6  1997/02/03 00:59:12  steve
 *  Rewrite the kit to use the Xform interface.
 *
 * Revision 1.5  1997/01/21 07:10:01  steve
 *  Separate the Tk support from the Tcl support,
 *  and add the ik_create command so that tclsh
 *  scripts can use ikit.
 *
 * Revision 1.4  1997/01/21 03:30:20  steve
 *  Add the ik_store command, and the ability for
 *  TIFF files to be stored. Also, add a preferences
 *  menu item and functions to populate it.
 *
 * Revision 1.3  1997/01/09 03:16:12  steve
 *  Allow for huge colormaps.
 *
 * Revision 1.2  1997/01/03 06:14:32  steve
 *  Support Color Palette display
 *
 * Revision 1.1.1.1  1996/12/22 05:25:56  steve
 * Start IKit project
 *
 */

# include  <tcl.h>

/*
 * the Ik_Image represents a single ik_image object. The image kit
 * passes a pointer to this opaque type around to the functions and
 * drivers that handle images for the ik_image.
 */

struct Ik_Image;
struct Ik_ImageFile;

/*
 * Find an ik_image with the given name, and return the token needed
 * to manipulate it with the operations below.
 */
struct Ik_Image* Ik_FindImage(const char*name);

/*
 * The image file, and possibly other sources, pass image data to the
 * ik_image with this function. The width and the height are the
 * dimensions in pixels, and the warp is the size of a line in
 * shorts. The pixel has some number of planes, each plane represented
 * by a 16 bit sample. So, grey would have 1 plane and each grey pixel
 * one byte, RGB color 3planes to make 6bytes per pixel.
 *
 * Once data is passed to the ik_image, it assumes the responsibility
 * of freeing it when it is no longer needed. The image data must
 * therefore be in malloc()ed memory.
 *
 * Ik_SetImage sets a 16bpi image with all the dimensions
 * specified. The palette is left uncalculated, because it may be
 * huge.
 *
 * Ik_SetImageBinary assumes that the incoming data is 1bpi and the
 * colors are black and white. The palette is configured
 * accordingly. The warp still specifies the number of 16BIT WORDS so
 * will probably be smaller then the width. Currently, only one plane
 * is supported. The highest order bit is rendered before the low
 * order bit.
 *
 * The Ik_GetImage retrieves all that information about the image. The
 * Ik_GetImageInfo is the same, but doesn't get the image data. The
 * latter along with the transform functions below are the preferred
 * interface. 
 */
extern void Ik_SetImage(struct Ik_Image*ik,
			unsigned short*data, unsigned planes,
			unsigned width, unsigned height, unsigned warp);

extern void Ik_SetImageBinary(struct Ik_Image*ik,
			      unsigned short*data, unsigned planes,
			      unsigned width, unsigned height, unsigned warp);

extern void Ik_GetImageInfo(const struct Ik_Image*ik, unsigned*planes,
			    unsigned*width, unsigned*height);

extern unsigned Ik_GetImageBPI(const struct Ik_Image*ik);

/*
 * These are types of transforms that can be applied to images. In
 * every case, the source image is clipped by the destination image
 * which is placed over the source image by the off_x and off_y
 * parameters. If there are two or more source images, they must have
 * the same dimensions (they can have different depths).
 *
 * In all cases, if the dst image is 0, it iwll not be tracked and the
 * off_x and off_y parameters will be ignored. The scan dimensions
 * will come from the source image(s) in this case.
 *
 * The source images will always be presented as 16bpi, so that is the
 * only density the processing function may function. The destination
 * image line points to the beginning of the data and it is up to the
 * processing function to get this right. This is a very reasonable
 * requirement as the processing usually know a priori what the
 * density should be.
 *
 * The RAW functions will present the data in different formats. In
 * this cate, the bpi field will give the format used by the source.
 */

struct ik_context {
      unsigned width;  /* destination image width */
      unsigned bpi;    /* destination bits-per-intensity */

      unsigned s_planes[2]; /* planes in the source image */
      unsigned s_bpi[2];    /* bits per intensity in the source image */
};

typedef void (*ik_1op_f)(unsigned short*d,
			 const unsigned short*s,
			 struct ik_context*ctx, void*cd);

/* Iterate over points, one source, no surrounding context. */
extern void Ik_point_1op(struct Ik_Image*dst, struct Ik_Image*src,
			 unsigned off_x, unsigned off_y,
			 ik_1op_f fun, void*cd);

/* Iterate all over lines, one source, no surrounding context. */
extern void Ik_line_1op(struct Ik_Image*dst, struct Ik_Image*src,
			unsigned off_x, unsigned off_y,
			ik_1op_f fun, void*cd);

/* Iterate all over lines, one source, no surrounding context. */
extern void Ik_line_1raw(struct Ik_Image*dst, struct Ik_Image*src,
			 ik_1op_f fun, void*cd);

typedef void (*ik_2op_f)(unsigned short*d,
			 const unsigned short*s1, const unsigned short*s2,
			 struct ik_context*ctx, void*cd);

/* Iterate over points, two sources, no surrounding context. */
extern void Ik_point_2op(struct Ik_Image*dst, struct Ik_Image*src1,
			 struct Ik_Image*src2,
			 unsigned off_x, unsigned off_y,
			 ik_2op_f fun, void*cd);

/*
 * These support operators that have some context from the source
 * images. The response to the callback is an array of line
 * pointers. The line being processed is at s[0]. Previous context is
 * at s[-1] etc, and later lines at s[1] etc. If the source image runs
 * out before or after, the s[-1] or s[1] pointer will as appropriate
 * be nil. It is up to the user to do the right thing.
 */

typedef void (*ik_n1op_f)(unsigned short*d,
			 const unsigned short*const*s,
			 struct ik_context*ctx, void*cd);

/* Iterate over lines, 1 source, up and down context. */
extern void Ik_nline_1op(struct Ik_Image*dst, struct Ik_Image*src1,
			 unsigned off_x, unsigned off_y,
			 unsigned con_up, unsigned con_do,
			 ik_n1op_f fun, void*cd);


/*
 * These methods implement support for color maps and palettes.
 *
 * The make_colormap() function remakes the color map being stored
 * with the image and allows the other functions to work. The
 * free_colormap() function deletes any memory associated with the
 * colormap.
 *
 * Every color is represented as a pointer to an array of unsigned
 * short integers. The size of the color is implied by the number of
 * planes in the colormap. For example, an RGB image has 3 intensities.
 *
 * The rewind_colormap returns a pointer to the first intensity of the
 * first color in the colormap, and the next_colormap function goes to
 * the next color. These are intended to support iterating through the
 * colormap of the image. Every color in the map also has a small
 * integer value (less then the number of colors) that various
 * functions can use to form maps. The count_colormap returns the
 * number of unique colors in the image.
 */

extern void Ik_make_colormap(struct Ik_Image*ik);
extern void Ik_free_colormap(struct Ik_Image*ik);


extern unsigned Ik_find_colormap(struct Ik_Image*ik, const unsigned short*cp);
extern unsigned Ik_find_colormap1(struct Ik_Image*ik, const unsigned short*cp);

extern unsigned Ik_count_colormap(struct Ik_Image*ik);
extern unsigned short*Ik_rewind_colormap(struct Ik_Image*ik, unsigned*i);
extern unsigned short*Ik_next_colormap(struct Ik_Image*ik, unsigned*i);

/*
 * This function gets the value of a specific pixel in the image. The
 * caller must supply memory for the intensities (as an array of
 * shorts) and pass the size of that memory. The call will then return
 * the number of intensities actually included in the pixel.
 */
extern unsigned Ik_GetPixel(struct Ik_Image*ik, unsigned x, unsigned y,
			    unsigned short*cp, unsigned ncp);

/*
 * These methods allow an application to create a color map without
 * scanning the image, or to scan the colors while reading them from a
 * source. It is an optimization.
 */
struct Ik_colormap;
extern struct Ik_colormap*Ik_alloc_colormap(unsigned planes);
extern void Ik_add_to_colormap(struct Ik_colormap*map,
			       const unsigned short*cp);
extern void Ik_set_colormap(struct Ik_Image*ik, struct Ik_colormap*map);



/*
 * Use this method to add a new image file type to the ik_image. The
 * type pointer contains all the information necessary for the
 * ik_image to do what it needs to do to get or store the image.
 */
extern void Ik_CreateImageFile(struct Ik_ImageFile*type);
extern struct Ik_ImageFile*Ik_GetImageFile(const char*name);

/*
 * The read function for the ImageFile takes an ik pointer and a list
 * of arguments and reads the file. the arguments come from the image
 * create command line and are used to identify the file, and other
 * things.
 */
typedef int (*Ik_ImageFileOp)(Tcl_Interp*interp,
			      struct Ik_Image*ik,
			      int argc, char*argv[]);

/*
 * This structure represents a handle to an image file type. The
 * Ik_CreateImageFile above uses this structure to carry the methods
 * to the ikit core.
 *
 * the ImageFile interface is also used to create pseudo-sources, such
 * as different kinds of images, image processing operators,
 * etc. Every image type should support the read.
 *
 * Some image formats also support write, for example to write out a
 * TIFF file. In that case, the write method will be defined. Image
 * processing operators in general will NOT support write.
 */
struct Ik_ImageFile {
	/* Name of the file format */
      const char*type_name;

	/* Function to read the file image into memory. */
      Ik_ImageFileOp read;
      Ik_ImageFileOp write;

	/* Initialize the following to 0. ik_image uses them internally. */

      struct Ik_ImageFile*next;
};


/*
 * These are handy routines for reading matrices from Tcl array
 * variables. Every matrix has x and y dimensions, and the values are
 * stored in a single stretch of memory in raster order. If there is a
 * problem reading the matrix, the function returns 0 and an error
 * message is left in the interp.
 *
 * The array contains a specific set of elements to define the
 * matrix. If the array name is m, the elements are:
 *   m(x)   -- width of matrix
 *   m(y)   -- height of matrix
 *   m(a,b) -- value of matrix at coordinate a b, where a and b are
 *             unsigned integers where a<x and b<y.
 *
 * The matrix value returned is allocated by malloc() and must be
 * released by free() when no longer needed.
 */
extern long* matrix_l(Tcl_Interp*interp, const char*name,
		      unsigned*m_wid, unsigned*m_hei);

#endif
